package AmuseWikiFarm::Controller::API;
use Moose;
use namespace::autoclean;

BEGIN { extends 'Catalyst::Controller'; }

use AmuseWikiFarm::Log::Contextual;

=head1 NAME

AmuseWikiFarm::Controller::API - Catalyst Controller

=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut



=head2 index

=cut

sub api :Chained('/site') :CaptureArgs(0) {
    my ($self, $c) = @_;
}

sub autocompletion :Chained('api') :Args(1) {
    my ($self, $c, $type) = @_;
    my $query = lc($type);
    my @list;
    if ($type =~ m/(topic|author)/) {
        my $type = $1;
        # include the deferred
        @list = map { $_->{name} }
          @{$c->stash->{site}->categories->with_texts(deferred => 1)->by_type($type)->listing_tokens};
    }
    elsif ($type eq 'adisplay') {
        @list = @{$c->stash->{site}->titles->list_display_authors};
    }
    Dlog_debug { "Found list for $type:  $_" } \@list;
    $c->stash(json => \@list);
    $c->detach($c->view('JSON'));
}

sub lexicon :Chained('api') :PathPart('lexicon.json') :Args(0) {
    my ($self, $c) = @_;
    my %out = (
               InvalidFileExtensionError => $c->loc("File format not allowed"),
               InvalidFileTypeError => $c->loc("File format not allowed"),
               MaxFileSizeError => $c->loc("File too big"),
               RequestError => $c->loc("Request failed! Please report the the problem"),
              );
    foreach my $js_string (
                           'Unused attachment',
                           'Remove',
                           'Insert the file into the body',
                           'Use the image as cover',
                           'File already in the body',
                           'Image already set as cover',
                           'Insert the file into the body at the cursor position',
                           'Please remove this file from the body first',
                           'Use the image as cover',
                          ) {
        $out{$js_string} = $c->loc($js_string);
    }
    $c->stash(json => \%out);
    $c->detach($c->view('JSON'));
}

sub ckeditor :Chained('api') :Args(0) {
    my ($self, $c) = @_;
    my $lang = $c->stash->{current_locale_code} || 'en';
    my $config = {
                  language => $lang,
                  toolbar => 'AmuseWiki',
                  toolbar_AmuseWiki => [
                                        {
                                         name => 'document',
                                         items => [ 'Source'],
                                        },
                                        {
                                         name =>  'clipboard',
                                         items  =>  [
                                                     'Cut',
                                                     'Copy',
                                                     'Paste',
                                                     'PasteText',
                                                     'PasteFromWord',
                                                     '-',
                                                     'Undo',
                                                     'Redo'
                                                    ],
                                        },
                                        {
                                         name =>  'editing',
                                         items  =>  [
                                                      'Find',
                                                      'Replace',
                                                      '-',
                                                      'SelectAll',
                                                     ],
                                        },
                                        {
                                         name =>  'basicstyles',
                                         items  =>  [
                                                     'Bold',
                                                     'Italic',
                                                     '-',
                                                     'RemoveFormat'
                                                    ],
                                        },
                                        {
                                         name => 'paragraph',
                                         items  => [
                                                    'NumberedList',
                                                    'BulletedList',
                                                    '-',
                                                    'Blockquote',
                                                   ],
                                        },
                                        {
                                         name =>  'styles',
                                         items  => [ 'Format'],
                                        }
                                       ],
                 };
    $c->stash(json => $config);
    $c->detach($c->view('JSON'));
}

=head1 AUTHOR

Marco Pessotto <melmothx@gmail.com>

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
